//****************************************************************************
//
// PRO : CORIOLIS
// MOD : CoAppApexCycleDecoder 
// ROL : This class implements the processing for the Apex cycle decoder
//		 application
//       - to read a given file containing one cycle of one Apex profiler
//       - to read the argos messages in it
//       - to decod the Apex frames
//       - write the decoded records in an ASCII output file
//
// CRE : 20/04/2001
// AUT : ST
// VER : @(#)1.16
//       @(#)03/12/22
//
//****************************************************************************
// 
// HST : 20/04/2001 ST  Creation
//		 26/06/2001 ST  Profile date QC is 0
//		 27/12/2001 ST  Now using the XML report
//		 04/01/2002 ST  CoDatProfile -> CoDatImmProfile
//		 11/02/2002 ST  Do not use CoStcTPtrSlist any more
//		 22/02/2002 ST  Use CoStcTools instead of CoStcTime
//		 26/02/2002 ST  Use CoDatPosition instead of CoStcLocalization
//		 09/04/2002 ST  Manage versions of Apex profilers
//		 12/08/2002 ST  Now use CoDatRecord
//		 13/08/2002 ST  FAE 205 : Manage double in Argos messages
//		 11/10/2002 ST  Rewriting for future Coriolis V2 compatibility
//		 02/12/2002 ST  Coriolis V2 compatibility
//		 09/10/2003 ST  Manage Apex V11.0
//		 22/12/2003 ST  Use ApexSuperFrame to generalize processing
//
//****************************************************************************
//
// Copyright, IFREMER, 2001
//
//****************************************************************************
static const char CoAppApexCycleDecoder_IDENT[] = "@(#) CORIOLIS : CoAppApexCycleDecoder.cpp 1.16 03/12/22";


//----------------------------------------------------------------------------
// INTERFACE
//----------------------------------------------------------------------------
#define _CoAppApexCycleDecoder_IN_
#include <CoAppApexCycleDecoder.h>
#undef  _CoAppApexCycleDecoder_IN_
#include <CoDfaArgosFile.h>
#include <CoDbaConnection.h>
#include <CoDbaPlatformAccessor.h>
#include <CoDfaDecodedCycleFile.h>
#include <CoDatApexFrame.h>
#include <CoDatArgosMessage.h>
#include <CoDatAttribute.h>
#include <CoDatLocatedRecord.h>
#include <CoDatTechnicalStatus.h>
#include <CoStcCatManager.h>
#include <CoStcError.h>
#include <CoStcFileConf.h>
#include <CoStcReport.h>
#include <CoStcTools.h>
#include <stdlib.h>




//****************************************************************************
//
// SPG : CoAppApexCycleDecoder
// ROL : Constructor
//
//****************************************************************************
CoAppApexCycleDecoder::
CoAppApexCycleDecoder(
    )
	: CoAppProcess()
	, _inputFileName("")
	, _outputFileName("")
	, _ignoreCrcErrors(FALSE)
	, _q_dbConnection((CoDbaConnection*)0)
	, _q_platformAccessor((CoDbaPlatformAccessor*)0)
	, _argosMessages()
	, _platform()
	, _version(0.0)
	, _technicalStatus()
	, _frames()
	, _superFrame()
	, _ascentProfile()
	, _immersionDrift()
	, _surfaceDrift()
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Check the parent instance validity
		//********************************************************************

		if (_isBad == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 108", 1000);
			CO_THROW(CoStc_k_error);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 119", 1001);
		_isBad = TRUE;
	}
}




//****************************************************************************
//
// SPG : ~CoAppApexCycleDecoder
// ROL : Destructor
//
//****************************************************************************
CoAppApexCycleDecoder::
~CoAppApexCycleDecoder(
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : Run
// ROL : Run the processing
//
//****************************************************************************
void
CoAppApexCycleDecoder::
Run(
    int p_argc,								// I : Arguments count
    char** qqp_argv							// I : Arguments array
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_optionCounter = 0;				// Loop counter
	RWCString l_currentOption = "";			// the current option
	size_t l_index = RW_NPOS;				// string index

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Check the number of parameters and decode them
		//********************************************************************

		if (p_argc < 3)
		{
			CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 187", 1002);
			CO_THROW(CoStc_k_error);
		}
		_inputFileName = qqp_argv[1];
		_outputFileName = qqp_argv[2];
		for (l_optionCounter = 4; l_optionCounter < p_argc; l_optionCounter++)
		{
			l_currentOption = qqp_argv[l_optionCounter];
			if (CoStcTools_Match(l_currentOption, "-ignore_crc", TRUE) == TRUE)
			{
				_ignoreCrcErrors = TRUE;
			}
			else
			{
				CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 201", 1003, l_currentOption);
				CO_THROW(CoStc_k_error);
			}
		}

		//********************************************************************
		// Run the life cycle
		//********************************************************************

		CoAppProcess::Run();
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 219", 1004);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Initialize
// ROL : Initialize the processing
//
//****************************************************************************
void
CoAppApexCycleDecoder::
Initialize(
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Create the database connection and the database accessors
		//********************************************************************

		_q_dbConnection = new CoDbaConnection();
		if (_q_dbConnection->IsBad() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 255", 1005);
			CO_THROW(CoStc_k_error);
		}
		_q_platformAccessor = new CoDbaPlatformAccessor(_q_dbConnection);
		if (_q_platformAccessor->IsBad() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 261", 1006);
			CO_THROW(CoStc_k_error);
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 272", 1007);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Compute
// ROL : Realize the processing
//
//****************************************************************************
void
CoAppApexCycleDecoder::
Compute(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Extract the argos messages from the input file
		//********************************************************************

		ExtractArgosMessages();

		//********************************************************************
		// Retrieve informations about the current profiler
		//********************************************************************

		RetrieveProfilerInfo();

		//********************************************************************
		// Mark the Argos messages in double
		//********************************************************************

		MarkDoubleArgosMessages();

		//********************************************************************
		// Extract the apex's data from the argos messages and instanciate
		// the corresponding frame objects
		//********************************************************************

		ExtractApexFrames();

		//********************************************************************
		// Compute and update the transmission statistics and the Argos
		// statistics
		//********************************************************************

		UpdateTransmissionStatistics();
		UpdateArgosStatistics();

		//********************************************************************
		// Sort and filter the list of frames
		//********************************************************************

		FilterFrames();

		//********************************************************************
		// Create the super frame
		//********************************************************************

		CreateSuperFrame();

		//********************************************************************
		// Fill the profiles
		//********************************************************************

		FillAscentProfile();
		FillImmersionDrift();

		//********************************************************************
		// Fill the technical status
		//********************************************************************

		FillTechnicalStatus();

		//********************************************************************
		// Write the decoded cycle file
		//********************************************************************

		WriteDecodedCycleFile();
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 372", 1008);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Finish
// ROL : Terminate the processing
//
//****************************************************************************
void
CoAppApexCycleDecoder::
Finish(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Destroy database objects
		//********************************************************************

		if (_q_platformAccessor != (CoDbaPlatformAccessor*)0)
		{
			delete _q_platformAccessor;
			_q_platformAccessor = (CoDbaPlatformAccessor*)0;
		}
		if (_q_dbConnection != (CoDbaConnection*)0)
		{
			delete _q_dbConnection;
			_q_dbConnection = (CoDbaConnection*)0;
		}

		//********************************************************************
		// Status is Ok
		//********************************************************************

		CoStc_g_report.AddStatus(TRUE);
		CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 1001),
								_platform.GetCode());
		CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 1002),
								_technicalStatus.GetAttribute("CYCLE_NUMBER").GetIntValue());
		CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 1003),
								_technicalStatus.GetAttribute("APX_ASCN_ProfLength").GetIntValue());
		CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 1004),
								_ascentProfile.GetRecordCount());
		CoStc_g_report.End();
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 438", 1009);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : ExtractArgosMessages
// ROL : Extract the argos messages from the input file
//
//****************************************************************************
void
CoAppApexCycleDecoder::
ExtractArgosMessages(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoDfaArgosFile l_argosFile;				// The input file
	RWCString l_beaconCode = "";			// The beacon code
	int l_messageCount = 0;					// number of Argos Messages
	int l_messageCounter = 0;				// loop counter

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Read the cycle file and memorize the Argos messages
		//********************************************************************

		l_argosFile.SetFileName(_inputFileName);
		l_argosFile.Read(_argosMessages);

		//********************************************************************
		// Check if there is at least one Argos message
		//********************************************************************

		if (_argosMessages.isEmpty() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 485", 1010);
			CO_THROW(CoStc_k_error);
		}

		//********************************************************************
		// Check if all the Argos messages are relative to the same profiler
		//********************************************************************

		l_beaconCode = _argosMessages.first()->GetBeaconCode();
		l_messageCount = _argosMessages.entries();
		for (l_messageCounter = 0; l_messageCounter < l_messageCount; l_messageCounter++)
		{
			if (_argosMessages.at(l_messageCounter)->GetBeaconCode() != l_beaconCode)
			{
				CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 499", 1011);
				CO_THROW(CoStc_k_error);
			}
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 511", 1012);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : RetrieveProfilerInfo
// ROL : Retrieve the info for the current profiler
//
//****************************************************************************
void
CoAppApexCycleDecoder::
RetrieveProfilerInfo(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWCString l_beaconCode = "";			// the platform beacon code

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Retrieve the beacon code and select the corresponding platform
		//********************************************************************

		l_beaconCode = _argosMessages.at(0)->GetBeaconCode();
		_q_platformAccessor->SelectByBeaconCode(l_beaconCode, _platform);

		//********************************************************************
		// Check the mandatory attributes
		//********************************************************************

		_platform.GetAttribute("PR_TYPE");
		_version = _platform.GetAttribute("PR_VERSION").GetDoubleValue();
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 563", 1013);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : MarkDoubleArgosMessages
// ROL : Mark the double in the Argos messages
//
//****************************************************************************
void
CoAppApexCycleDecoder::
MarkDoubleArgosMessages(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_messageCount = 0;					// number of Argos messages
	int l_messageCounter = 0;				// loop counter
	CoDatArgosMessage* ql_argosMessage =	// the current Argos message
		(CoDatArgosMessage*)0;
	CoDatTechnicalStatus l_dummyStatus;		// a dummy technical status
	CoDatLocatedRecord* ql_surfaceRecord =	// the current surface record
		(CoDatLocatedRecord*)0;
	RWTPtrSlist<CoDatApexFrame> l_frames;	// the list of Apex frames for the current message
	CoDatApexFrame* ql_frame =				// the current Apex frame
		(CoDatApexFrame*)0;
	RWCString l_frameId = "";				// id of the current frame
	RWTValSlist<RWCString> l_frameIds;		// the list of frame id

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// For each argos message (last to first)
		//********************************************************************

		l_messageCount = _argosMessages.entries();
		for (l_messageCounter = l_messageCount - 1; l_messageCounter >= 0; l_messageCounter--)
		{
			//****************************************************************
			// Get the list of Apex frames
			//****************************************************************

			ql_argosMessage = _argosMessages.at(l_messageCounter);
			ql_argosMessage->GetApexFrames(_platform,
										   l_dummyStatus,
										   ql_surfaceRecord,
										   l_frames);

			//****************************************************************
			// For each Apex frame
			//****************************************************************

			while (l_frames.isEmpty() == FALSE)
			{
				//************************************************************
				// Retrieve the current frame and compute it's id
				//************************************************************

				ql_frame = l_frames.removeFirst();
				l_frameId = RWCString(ql_frame->GetEmittingSatellite()) +
							CoStcTools_ToString(ql_frame->GetEmittingDateTime(), "-%d/%m/%Y %H:%M:%S");
				delete ql_frame;

				//************************************************************
				// If the id exists, mark the message
				//************************************************************

				if (l_frameIds.contains(l_frameId) == TRUE)
				{
					ql_argosMessage->SetDoubleFlag(TRUE);
					l_frames.clearAndDestroy();
				}
				else
				{
					l_frameIds.append(l_frameId);
				}
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 660", 1014);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : ExtractApexFrames
// ROL : Extract the Apex's datas from the argos messages and instanciate
//	 the corresponding frame objects
//
//****************************************************************************
void
CoAppApexCycleDecoder::
ExtractApexFrames(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_messageCount = 0;					// number of Argos messages
	int l_messageCounter = 0;				// loop counter
	CoDatArgosMessage* ql_argosMessage =	// the current Argos message
		(CoDatArgosMessage*)0;
	CoDatLocatedRecord* ql_surfaceRecord =	// the current surface record
		(CoDatLocatedRecord*)0;
	RWTPtrSlist<CoDatApexFrame> l_frames;	// the list of Apex frames for the current message
	CoDatApexFrame* ql_frame =				// the current Apex frame
		(CoDatApexFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Sort the Argos messages
		//********************************************************************

		CoStcTools_Sort(_argosMessages, CoDatArgosMessage::CompareDateTime);

		//********************************************************************
		// For each argos message
		//********************************************************************

		l_messageCount = _argosMessages.entries();
		for (l_messageCounter = 0; l_messageCounter < l_messageCount; l_messageCounter++)
		{
			//****************************************************************
			// Get the list of Apex frames
			//****************************************************************

			ql_argosMessage = _argosMessages.at(l_messageCounter);
			ql_argosMessage->GetApexFrames(_platform,
										   _technicalStatus,
										   ql_surfaceRecord,
										   l_frames);

			//****************************************************************
			// Append each frame to the global list
			//****************************************************************

			while (l_frames.isEmpty() == FALSE)
			{
				_frames.append(l_frames.removeFirst());
			}

			//****************************************************************
			// Memorize the surface record
			//****************************************************************

			if ((ql_surfaceRecord != (CoDatLocatedRecord*)0) &&
				(ql_argosMessage->GetDoubleFlag() == FALSE))
			{
				_surfaceDrift.AddRecord(ql_surfaceRecord);
			}
		}

		//********************************************************************
		// If there is no frame, there is something wrong
		//********************************************************************

		if (_frames.entries() == 0)
		{
			CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 749", 1015);
			CO_THROW(CoStc_k_error);
		}

		//********************************************************************
		// Sort the surface drift
		//********************************************************************

		_surfaceDrift.SortRecordsByDateTime();
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 766", 1016);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : UpdateTransmissionStatistics
// ROL : Update the transmission statistics
//
//****************************************************************************
void
CoAppApexCycleDecoder::
UpdateTransmissionStatistics(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_receivedFrames = 0;				// Number of received frames (without doubles)
	int l_completeFrames = 0;				// Number of complete frames (without doubles)
	int l_incompleteFrames = 0;				// Number of incomplete frames
	int l_crcOkFrames = 0;					// Number of crc ok frames (without doubles)
	int l_receivedTechnicalFrames = 0;		// Number of received technical frames (without doubles)
	int l_crcOkTechnicalFrames = 0;			// Number of crc ok technical frames (without doubles)
	int l_receivedAscentFrames = 0;			// Number of received ascent frames (without doubles)
	int l_completeAscentFrames = 0;			// Number of complete ascent frames (without doubles)
	int l_incompleteAscentFrames = 0;		// Number of complete ascent frames (without doubles)
	int l_crcOkAscentFrames = 0;			// Number of crc ok ascent frames (without doubles)
	int l_frameCount = 0;					// number of frames
	int l_frameCounter = 0;					// loop counter
	RWTime l_emittingDateTime =				// the current emitting date/time
		(unsigned long)0;
	char l_emittingSatellite ='\0';			// the current emitting satellite
	CoDatApexFrame* ql_frame =				// the current Apex frame
		(CoDatApexFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Initialize the counts
		//********************************************************************

		l_incompleteAscentFrames = _technicalStatus.GetAttribute("TRANS_APROF_Incompl").GetIntValue();
		l_incompleteFrames = _technicalStatus.GetAttribute("TRANS_OTHER_Incompl").GetIntValue() +
							 l_incompleteAscentFrames;
		l_receivedAscentFrames = l_incompleteAscentFrames;
		l_receivedFrames = l_incompleteFrames;

		//********************************************************************
		// Remove doubles from technical frames
		//********************************************************************

		l_frameCount = _frames.entries();
		if (l_frameCount != 0)
		{
			CoStcTools_Sort(_frames, CoDatDataFrame::CompareEmittingDateTimeAndSatellite);
			l_emittingDateTime = RWTime((unsigned long)0);
			l_emittingSatellite = '\0';
			for (l_frameCounter = 0; l_frameCounter < l_frameCount; l_frameCounter++)
			{
				ql_frame = _frames.at(l_frameCounter);
				if ((ql_frame->GetEmittingDateTime() != l_emittingDateTime) ||
					(ql_frame->GetEmittingSatellite() != l_emittingSatellite))
				{
					l_receivedFrames++;
					l_completeFrames++;
					if (ql_frame->GetFrameNumber() == 1)
					{
						l_receivedTechnicalFrames++;
					}
					else
					{
						l_receivedAscentFrames++;
						l_completeAscentFrames++;
					}
					if (ql_frame->IsCrcOk() == TRUE)
					{
						l_crcOkFrames++;
						if (ql_frame->GetFrameNumber() == 1)
						{
							l_crcOkTechnicalFrames++;
						}
						else
						{
							l_crcOkAscentFrames++;
						}
					}
					l_emittingDateTime = ql_frame->GetEmittingDateTime();
					l_emittingSatellite = ql_frame->GetEmittingSatellite();
				}
			}
		}

		//********************************************************************
		// Update the status
		//********************************************************************

		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Received", l_receivedFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Complete", l_completeFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_CRC_OK", l_crcOkFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Tech_frames", l_receivedTechnicalFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Tech_OK", l_crcOkTechnicalFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_APROF_Rcvd", l_receivedAscentFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_APROF_Complet", l_completeAscentFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_APROF_OK", l_crcOkAscentFrames));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 888", 1017);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : UpdateArgosStatistics
// ROL : Update the Argos statistics
//
//****************************************************************************
void
CoAppApexCycleDecoder::
UpdateArgosStatistics(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_messageCount = 0;					// Number of Argos messages
	int l_messageCounter = 0;				// Loop counter
	CoDatArgosMessage* ql_argosMessage =	// The current Argos message
		(CoDatArgosMessage*)0;
	CoDatLocatedRecord* ql_record =			// The surface record
		(CoDatLocatedRecord*)0;
	char l_class = '\0';					// current Argos class
	int l_argosClass1 = 0;					// number of class 1 messages
	int l_argosClass2 = 0;					// number of class 2 messages
	int l_argosClass3 = 0;					// number of class 3 messages
	int l_argosRejected = 0;				// number of rejected messages
	int l_argosTotal = 0;					// number of messages

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// For each Argos messages
		//********************************************************************

		l_messageCount = _argosMessages.entries();
		for (l_messageCounter = 0; l_messageCounter < l_messageCount; l_messageCounter++)
		{
			//****************************************************************
			// If the current Argos message is not the double for another one
			//****************************************************************

			ql_argosMessage = _argosMessages.at(l_messageCounter);
			if (ql_argosMessage->GetDoubleFlag() == FALSE)
			{
				//************************************************************
				// if the argos message has a surface record
				//************************************************************

				ql_record = ql_argosMessage->GetSurfaceRecord();
				if (ql_record != (CoDatLocatedRecord*)0)
				{
					//********************************************************
					// Depending of the current Argos class
					//********************************************************

					l_class = ql_record->GetPosition().GetClass();
					switch (l_class)
					{
					    case '1' :
							l_argosClass1++;
					    break;

					    case '2' :
							l_argosClass2++;
					    break;

					    case '3' :
							l_argosClass3++;
					    break;

					    default :
							l_argosRejected++;
					    break;
					}
				}

				//************************************************************
				// If the argos message hos no surface record
				//************************************************************

				else
				{
					l_argosRejected++;
				}
			}
		}

		//********************************************************************
		// Update the status
		//********************************************************************

		l_argosTotal = l_argosClass1 + l_argosClass2 + l_argosClass3 + l_argosRejected;
		_technicalStatus.AddAttribute(CoDatAttribute("POS_Class_1", l_argosClass1));
		_technicalStatus.AddAttribute(CoDatAttribute("POS_Class_2", l_argosClass2));
		_technicalStatus.AddAttribute(CoDatAttribute("POS_Class_3", l_argosClass3));
		_technicalStatus.AddAttribute(CoDatAttribute("POS_Rejected", l_argosRejected));
		_technicalStatus.AddAttribute(CoDatAttribute("POS_Total", l_argosTotal));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 1005", 1018);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : FilterFrames
// ROL : Filter the list of frames in order to remove doubles
//
//****************************************************************************
void
CoAppApexCycleDecoder::
FilterFrames(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWTPtrSlist<CoDatApexFrame> l_temp;		// temporary list
	RWTPtrSlist<CoDatApexFrame> l_final;	// the filtered list
	int l_counter = 0;						// loop counter
	int l_recombinedFrames = 0;				// Number of recombined frames
	int l_recombinedAscentFrames = 0;		// Number of recombined ascent frames

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Sort the frame list
		//********************************************************************

		CoStcTools_Sort(_frames, CoDatApexFrame::CompareFrameNumber);

		//********************************************************************
		// while the frame list is not empty
		//********************************************************************

		while (_frames.isEmpty() == FALSE)
		{
			//****************************************************************
			// Copy in the temp list all the frames which have the same number
			//****************************************************************

			l_temp.append(_frames.removeFirst());
			while ((_frames.isEmpty() == FALSE) &&
				   (_frames.first()->GetFrameNumber() == l_temp.first()->GetFrameNumber()))
			{
				l_temp.append(_frames.removeFirst());
			}

			//****************************************************************
			// Find the first frame which has a good Crc (if exists)
			//****************************************************************

			l_counter = 0;
			while ((l_counter < l_temp.entries()) &&
				   (l_temp.at(l_counter)->IsCrcOk() == FALSE))
			{
				l_counter++;
			}

			//****************************************************************
			// if a frame with a good Crc has been found
			//****************************************************************

			if (l_counter < l_temp.entries())
			{
				//************************************************************
				// Append this frame to the filtered list and destroy
				// other frames
				//************************************************************

				l_final.append(l_temp.removeAt(l_counter));
				l_temp.clearAndDestroy();
			}

			//****************************************************************
			// If no frame with a good Crc has been found
			//****************************************************************

			else
			{
				//************************************************************
				// If the list has at least three frames, try to reconstruct
				// a new one
				//************************************************************

				if (l_temp.entries() >= 3)
				{
					l_temp.first()->Recombined(l_temp);
					l_temp.first()->Check8BitsCrc();
					if (l_temp.first()->IsCrcOk() == TRUE)
					{
						l_final.append(l_temp.removeFirst());
						l_recombinedFrames++;
						if (l_final.last()->GetFrameNumber() != 1)
						{
							l_recombinedAscentFrames++;
						}
					}
					else
					{
						if (_ignoreCrcErrors == TRUE)
						{
							l_final.append(l_temp.removeFirst());
						}
					}
				}
				else
				{
					if (_ignoreCrcErrors == TRUE)
					{
						l_final.append(l_temp.removeFirst());
					}
				}
				l_temp.clearAndDestroy();
			} 
		}

		//********************************************************************
		// Update the input list
		//********************************************************************

		while (l_final.isEmpty() == FALSE)
		{
			_frames.append(l_final.removeFirst());
		}

   		//********************************************************************
		// If the first frame is not a technical frame : error
		//********************************************************************

		if (_frames.first()->GetFrameNumber() != 1)
		{
			CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 1147", 1019);
			CO_THROW(CoStc_k_error);
		}

		//********************************************************************
		// Fill the technical status
		//********************************************************************

		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_APROF_Emitted", int(_frames.last()->GetFrameNumber() - 1)));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_APROF_Distinc", int(_frames.entries() - 1)));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Recombined", l_recombinedFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_APROF_Recom", l_recombinedAscentFrames));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 1167", 1020);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : CreateSuperFrame
// ROL : Create the super frame which is the concatenation of all the Apex frames
//
//****************************************************************************
void
CoAppApexCycleDecoder::
CreateSuperFrame(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Set the version number
		//********************************************************************

		_superFrame.SetVersion(_version);

		//********************************************************************
		// And set the frames
		//********************************************************************

		_superFrame.Initialize(_frames);
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 1215", 1021);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : FillAscentProfile
// ROL : Fill the ascent profile
//
//****************************************************************************
void
CoAppApexCycleDecoder::
FillAscentProfile(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoStcFileConf l_conf;					// access to the configuration file
	char l_maxLocateClass = 0;				// the max acceptable location class
	int l_maxLocateSpan = 0;				// the max acceptable locate span
	RWTime l_profileTime =					// time of the profile
		(unsigned long)0;
	CoDatPosition l_position;				// position of the profile
	RWTPtrSlist<CoDatRecord> l_records;		// temporary list of records
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Retrieve the locate qc and the locate span from the conf file
		//********************************************************************

		l_maxLocateClass = char(atoi(l_conf.GetBlockValue("PROFILER_CYCLE_DECODER",
														  "MAX_LOCATE_QC"))) + '0';
		l_maxLocateSpan = atoi(l_conf.GetBlockValue("PROFILER_CYCLE_DECODER",
													"MAX_LOCATE_SPAN"));

		//********************************************************************
		// Compute the profile time
		//********************************************************************

		l_profileTime = _surfaceDrift.GetRecord(0)->GetDateTime();
		_ascentProfile.SetDateTime(l_profileTime);
		_ascentProfile.SetDateTimeQuality('0');

		//********************************************************************
		// Compute the profile position
		//********************************************************************

		ComputeProfilePosition(_surfaceDrift,
							   l_profileTime,
							   l_maxLocateClass,
							   l_maxLocateSpan,
							   l_position);
		_ascentProfile.SetPosition(l_position);

		//********************************************************************
		// Get the records
		//********************************************************************

		_superFrame.GetAscentRecords(l_records);

		//********************************************************************
		// For each record of the frame
		//********************************************************************

		while (l_records.entries() != 0)
		{
			//****************************************************************
			// Retrieve the record
			//****************************************************************

			ql_record = l_records.removeFirst();

			//****************************************************************
			// Compute the density
			//****************************************************************

			ql_record->ComputeAndAddDensMeasure();

			//****************************************************************
			// And append the record to the profile
			//****************************************************************

			_ascentProfile.AddRecord(ql_record);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 1320", 1022);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : FillImmersionDrift
// ROL : Fill the immersion drift
//
//****************************************************************************
void
CoAppApexCycleDecoder::
FillImmersionDrift(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_cyclePeriod = 0;					// the cycle period (in days)
	int l_driftSamplingPeriod = 0;			// the immersion drift sampling period (in minutes)
	RWTime l_profileTime =					// time of the profile
		(unsigned long)0;
	RWTPtrSlist<CoDatRecord> l_records;		// temporary list of records
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Depending of the profiler version
		//********************************************************************

		if (_version == 10.0)
		{
			//****************************************************************
			// Retrieve the cycle period and the immersion drift sampling period
			//****************************************************************

			l_cyclePeriod = _platform.GetAttribute("PR_CYCLE_PERIOD").GetIntValue();
			l_driftSamplingPeriod = _platform.GetAttribute("PR_IMMERSION_DRIFT_PERIOD").GetIntValue();

			//****************************************************************
			// Compute the profile time
			//****************************************************************

			l_profileTime = _surfaceDrift.GetRecord(_surfaceDrift.GetRecordCount() - 1)->GetDateTime();
			l_profileTime -= l_cyclePeriod * 24 * 60 * 60;
			l_profileTime = l_profileTime + 6 * 60 * 60;
			l_profileTime = RWTime(long(rint(double(l_profileTime.seconds()) / 3600.0)) * 3600);
			_immersionDrift.SetDateTime(l_profileTime);
			l_profileTime = l_profileTime + 8 * 60 * 60;
			_immersionDrift.SetDateTimeQuality('0');

			//****************************************************************
			// Get the records
			//****************************************************************

			_superFrame.GetImmersionRecords(l_records);

			//****************************************************************
			// For each record of the frame
			//****************************************************************

			while (l_records.entries() != 0)
			{
				//************************************************************
				// Retrieve the record
				//************************************************************

				ql_record = l_records.removeFirst();

				//************************************************************
				// Compute the density
				//************************************************************

				ql_record->ComputeAndAddDensMeasure();

				//************************************************************
				// Set it's date-time
				//************************************************************

				ql_record->SetDateTime(l_profileTime);
				l_profileTime += l_driftSamplingPeriod * 60;

				//************************************************************
				// And append the record to the profile
				//************************************************************

				_immersionDrift.AddRecord(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 1428", 1023);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : FillTechnicalStatus
// ROL : Fill the technical status
//
//****************************************************************************
void
CoAppApexCycleDecoder::
FillTechnicalStatus(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Retrieve the technical status
		//********************************************************************

		_superFrame.GetTechnicalStatus(_technicalStatus);
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 1470", 1024);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : WriteDecodedCycleFile
// ROL : Initialize and write the decoded cycle file
//
//****************************************************************************
void
CoAppApexCycleDecoder::
WriteDecodedCycleFile(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoDfaDecodedCycleFile l_file;			// the decoded cycle file
	CoStcFileConf l_fileConf;				// Access to the configuration file
	RWCString l_decoderVersion = "";		// the decoder version

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Set the output file name
		//********************************************************************

		l_file.SetFileName(_outputFileName);

		//********************************************************************
		// Set the decoder version
		//********************************************************************

		l_decoderVersion = l_fileConf.GetBlockValue("PROFILER_CYCLE_DECODER", "DECODER_VERSION");
		l_file.SetDecoderVersion(l_decoderVersion);

		//********************************************************************
		// Write the content of the file
		//********************************************************************

		l_file.Write(_platform,
					 CoDatProfile(),
					 _immersionDrift,
					 _ascentProfile,
					 _surfaceDrift,
					 _technicalStatus);
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 1534", 1025);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : ComputeProfilePosition
// ROL : Compute the position of a profile in a trajectory with criteria
//
//****************************************************************************
/*static*/ void
CoAppApexCycleDecoder::
ComputeProfilePosition(
	const CoDatProfile& p_trajectory,		// I : the trajectory
	const RWTime& p_profileDateTime,		// I : the profile date-time
	char p_maxLocateClass,					// I : the max acceptable location class
	int p_maxLocateSpan,					// I : the max acceptable locate span
	CoDatPosition& p_position				// O : the profile position
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_recordCount = 0;					// number of records
	int l_recordCounter = 0;				// loop counter
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Initialize the return psotion
		//********************************************************************

		p_position = CoDatPosition();
		p_position.SetQuality('9');

		//********************************************************************
		// Compute the profile position, ie find the first valid surface
		// position
		//********************************************************************

		l_recordCount = p_trajectory.GetRecordCount();
		for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
		{
			ql_record = p_trajectory.GetRecord(l_recordCounter);
			if (ql_record->GetPosition().GetClass() >= p_maxLocateClass)
			{
				p_position = ql_record->GetPosition();
				if (ql_record->GetDateTime() > p_profileDateTime + p_maxLocateSpan * 60)
				{
					p_position.SetQuality('3');
				}
				else
				{
					p_position.SetQuality('1');
				}
				l_recordCounter = l_recordCount;
			}
		}

		//********************************************************************
		// If the position was not found, use the best position available
		//********************************************************************

		if ((p_position.GetLatitude() == CoDat_k_undefinedDouble) &&
			(l_recordCount != 0))
		{
			p_position.SetClass('0');
			for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
			{
				ql_record = p_trajectory.GetRecord(l_recordCounter);
				if (ql_record->GetPosition().GetClass() > p_position.GetClass())
				{
					p_position = ql_record->GetPosition();
				}
			}
			p_position.SetQuality('3');
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppApexCycleDecoder.cpp 1.16 1630", 1026);
		CO_RETHROW;
	}
}






