//****************************************************************************
//
// PRO : CORIOLIS
// MOD : CoAppProfilerCycleDecoder 
// ROL : This class implements the processing for the Profiler cycle decoder
//		 application
//       - to read a given file containing one cycle of one profiler
//       - to read the argos messages in it
//       - to decod the profiler frames
//       - write the decoded records in an ASCII output file
//
// CRE : 20/04/2001
// AUT : ST
// VER : @(#)1.7
//       @(#)02/12/02
//
//****************************************************************************
// 
// HST : 20/04/2001 ST  Creation
//		 26/06/2001 ST  Profile date QC is 0
//		 27/12/2001 ST  Now using the XML report
//		 11/02/2002 ST  Do not use CoStcTPtrSlist any more
//		 27/08/2002 ST  Manage the metocean floats
//		 07/10/2002 ST  GetProfilerTypeAndVersion replace GetProfilerType
//		 02/12/2002 ST  Coriolis V2 compatibility
//
//****************************************************************************
//
// Copyright, IFREMER, 2001
//
//****************************************************************************
static const char CoAppProfilerCycleDecoder_IDENT[] = "@(#) CORIOLIS : CoAppProfilerCycleDecoder.cpp 1.7 02/12/02";


//----------------------------------------------------------------------------
// INTERFACE
//----------------------------------------------------------------------------
#define _CoAppProfilerCycleDecoder_IN_
#include <CoAppProfilerCycleDecoder.h>
#undef  _CoAppProfilerCycleDecoder_IN_
#include <iostream.h>
#include <CoAppApexCycleDecoder.h>
#include <CoAppMetoceanCycleDecoder.h>
#include <CoAppProvorCycleDecoder.h>
#include <CoDfaArgosFile.h>
#include <CoDbaConnection.h>
#include <CoDbaPlatformAccessor.h>
#include <CoDatArgosMessage.h>
#include <CoDatPlatform.h>
#include <CoStcCatManager.h>
#include <CoStcError.h>
#include <CoStcReport.h>
#include <rw/tpslist.h>




//****************************************************************************
//
// SPG : CoAppProfilerCycleDecoder
// ROL : Constructor
//
//****************************************************************************
CoAppProfilerCycleDecoder::
CoAppProfilerCycleDecoder(
    )
	: CoAppProcess()
	, _inputFileName("")
	, _argc(0)
	, _qq_argv((char**)0)
	, _q_dbConnection((CoDbaConnection*)0)
	, _q_platformAccessor((CoDbaPlatformAccessor*)0)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Check the parent instance validity
		//********************************************************************

		if (_isBad == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 90", 2401);
			CO_THROW(CoStc_k_error);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 101", 2402);
		_isBad = TRUE;
	}
}




//****************************************************************************
//
// SPG : ~CoAppProfilerCycleDecoder
// ROL : Destructor
//
//****************************************************************************
CoAppProfilerCycleDecoder::
~CoAppProfilerCycleDecoder(
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : Run
// ROL : Run the processing
//
//****************************************************************************
void
CoAppProfilerCycleDecoder::
Run(
    int p_argc,								// I : Arguments count
    char** qqp_argv							// I : Arguments array
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Output the xml header report
		//********************************************************************

		CoStc_g_report.Begin(CoStc_g_catManager.GetInfo(1, 2402),
							 CoStc_g_catManager.GetInfo(1, 2403));

		//********************************************************************
		// Check the number of parameters and decode them
		//********************************************************************

		switch (p_argc)
		{
			case 3 :
			{
				_inputFileName = qqp_argv[1];
			}
			break;

			case 4 :
			{
				if (RWCString(qqp_argv[1]) == "-i")
				{
					_inputFileName = qqp_argv[2];
				}
				else
				{
					CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 186", 2403);
					CO_THROW(CoStc_k_error);
				}
			}
			break;

			default :
			{
				CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 194", 2403);
				CO_THROW(CoStc_k_error);
			}
		}
		_argc = p_argc;
		_qq_argv = qqp_argv;

		//********************************************************************
		// Run the life cycle
		//********************************************************************

		CoAppProcess::Run();
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_report.AddStatus(FALSE);
		CoStc_g_report.End();
		CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 216", 2404);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Initialize
// ROL : Initialize the processing
//
//****************************************************************************
void
CoAppProfilerCycleDecoder::
Initialize(
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Create the database connection and the database accessors. This job
		// could be done later, but it is done here to isolate everything
		// concerning the database
		//********************************************************************

		_q_dbConnection = new CoDbaConnection();
		if (_q_dbConnection->IsBad() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 254", 2405);
			CO_THROW(CoStc_k_error);
		}
		_q_platformAccessor = new CoDbaPlatformAccessor(_q_dbConnection);
		if (_q_platformAccessor->IsBad() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 260", 2406);
			CO_THROW(CoStc_k_error);
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 271", 2407);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Compute
// ROL : Realize the processing
//
//****************************************************************************
void
CoAppProfilerCycleDecoder::
Compute(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWCString l_profilerType = "";			// The profiler type ("PROVOR", "APEX")
	double l_profilerVersion = 0.0;			// the profiler version
	CoAppProvorCycleDecoder l_provorDecoder;	// the provor decoder
	CoAppApexCycleDecoder l_apexDecoder;	// the apex decoder
	CoAppMetoceanCycleDecoder l_metoceanDecoder;	// the metocean decoder

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Extract the argos messages from the input file
		//********************************************************************

		GetProfilerTypeAndVersion(l_profilerType, l_profilerVersion);

		//********************************************************************
		// If the profiler is a PROVOR profiler
		//********************************************************************

		if (l_profilerType == "PROVOR")
		{
			l_provorDecoder.Run(_argc, _qq_argv);
		}

		//********************************************************************
		// If the profiler is an APEX profiler
		//********************************************************************

		else if (l_profilerType == "APEX")
		{
			l_apexDecoder.Run(_argc, _qq_argv);
		}

		//********************************************************************
		// If the profiler is a METOCEAN profiler
		//********************************************************************

		else if (l_profilerType == "METOCEAN")
		{
			l_metoceanDecoder.Run(_argc, _qq_argv);
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 346", 2408);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Finish
// ROL : Terminate the processing
//
//****************************************************************************
void
CoAppProfilerCycleDecoder::
Finish(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Destroy database objects
		//********************************************************************

		if (_q_platformAccessor != (CoDbaPlatformAccessor*)0)
		{
			delete _q_platformAccessor;
			_q_platformAccessor = (CoDbaPlatformAccessor*)0;
		}
		if (_q_dbConnection != (CoDbaConnection*)0)
		{
			delete _q_dbConnection;
			_q_dbConnection = (CoDbaConnection*)0;
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 397", 2409);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : GetProfilerTypeAndVersion
// ROL : Extract the argos messages from the input file and retrieve
//		 the type of the corresponding profiler
//
//****************************************************************************
void
CoAppProfilerCycleDecoder::
GetProfilerTypeAndVersion(
	RWCString& p_profilerType,				// O : the profiler type
	double& p_profilerVersion				// O : the profiler version
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoDfaArgosFile l_argosFile;				// The input file
	RWTPtrSlist<CoDatArgosMessage> l_argosMessages;	// The list of the Argos messages
	RWCString l_beaconCode = "";			// The platform beacon code
	CoDatPlatform l_platform;				// The platform

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Read the cycle file and memorize the Argos messages
		//********************************************************************

		l_argosFile.SetFileName(_inputFileName);
		l_argosFile.Read(l_argosMessages);

		//********************************************************************
		// Check if there is at least one Argos message
		//********************************************************************

		if (l_argosMessages.isEmpty() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 447", 2410);
			CO_THROW(CoStc_k_error);
		}

		//********************************************************************
		// Retrieve the beacon code
		//********************************************************************

		l_beaconCode = l_argosMessages.first()->GetBeaconCode();

		//********************************************************************
		// Retrieve the corresponding platform
		//********************************************************************

		_q_platformAccessor->SelectByBeaconCode(l_beaconCode, l_platform);
		p_profilerType = l_platform.GetAttribute("PR_TYPE").GetStringValue();
		p_profilerVersion = l_platform.GetAttribute("PR_VERSION").GetDoubleValue();
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProfilerCycleDecoder.cpp 1.7 472", 2411);
		CO_RETHROW;
	}
}






