//****************************************************************************
//
// PRO : CORIOLIS 
// MOD : CoAppProfilerCycleDecoderMain
// ROL : Main entry for the profiler cycle decoder software. The purpose of
//		 this software is :
//		 - to read a given file containing one cycle of one profiler
//		 - to read the argos messages in it
//		 - to decod the profiler frames
//		 - write the decoded records in an ASCII output file
//
// CRE : 27/08/2001
// AUT : ST
// VER : @(#)1.5
//       @(#)02/10/11
//
//****************************************************************************
// 
// HST : 27/08/2001 ST  Creation
//		 27/12/2001 ST  Now using the XML report
//		 11/10/2002 ST  Now using the new configuration file syntax
//
//****************************************************************************
//
// Copyright, IFREMER, 2001
//
//****************************************************************************
static const char CoAppProfilerCycleDecoderMain_IDENT[] = "@(#) CORIOLIS : CoAppProfilerCycleDecoderMain.cpp 1.5 02/10/11";


//----------------------------------------------------------------------------
// INTERFACE
//----------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <rw/zone.h>
#include <CoAppProfilerCycleDecoder.h>
#include <CoAppBuildId.h>
#include <CoStcCatManager.h>
#include <CoStcError.h>
#include <CoStcFileConf.h>
#include <CoStcLogFile.h>
#include <CoStcReport.h>
#include <CoStcSignal.h>




//****************************************************************************
//
// SPG : ProcessSignal
// ROL : Application's exit when a signal is received
//
//****************************************************************************
static void
ProcessSignal(
	int p_signal				// I : The signal received
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	char ql_buffer[16];				// conversion buffer

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	//************************************************************************
	// Output the signal number and exit
	//************************************************************************

	(void)sprintf(ql_buffer, "%d", p_signal);
	CoStc_g_error.Send(1, "CoAppProfilerCycleDecoderMain.cpp 1.5 75", 2301, ql_buffer);

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_THROW(CoStc_k_error);
}




//****************************************************************************
//
// SPG : main
// ROL : Application initialization and processing instantiation
// 
//****************************************************************************
int
main(
	int p_argc,			// I : Arguments count
	char** qqp_argv		// I : Arguments array
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoStcSignal l_signal(ProcessSignal);	// Signals manager
	CoStcFileConf l_fileConf;				// Access to the configuration file
	RWCString l_errorPath = "";				// Path for the messages database
	RWCString l_language = "";				// Current language
	RWCString l_logFileName = "";			// Name of the log file
	CoStcLogFile* ql_logFile = 				// Log file for the error manager
		(CoStcLogFile*)0;
	CoAppProfilerCycleDecoder l_decoder; 	// The processing
	RWBoolean l_showMessage = TRUE;			// Shows the error message
    RWCString l_reportDirectory = "";		// Path for the xml reports
    
//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Display the build Id info string
		//********************************************************************

		CoApp_M_DisplayBuildId(p_argc, qqp_argv);

		//********************************************************************
		// Change the local zone
		//********************************************************************

		RWZone::local(&RWZone::utc());

		//********************************************************************
		// Initialization of the cat file manager
		//********************************************************************

		if (l_fileConf.IsBad() == TRUE)
		{
			CoStc_g_error.Print("[CoAppProfilerCycleDecoderMain.cpp 1.5 138] Can't access configuration file\n");
			CO_THROW(CoStc_k_error);
		}
		l_errorPath = l_fileConf.GetValue("CO_ERROR");
		l_language = l_fileConf.GetValue("CO_LANG");
		CoStc_g_catManager.Initialize(l_errorPath, l_language);

		//********************************************************************
		// Initialization of the log file for the error manager
		//********************************************************************

		l_logFileName = l_fileConf.GetBlockValue("PROFILER_CYCLE_DECODER", "LOG");
		ql_logFile = new CoStcLogFile(l_logFileName);
		if (ql_logFile->IsBad() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProfilerCycleDecoderMain.cpp 1.5 153", 2302);
			CO_THROW(CoStc_k_error);
		}
		CoStc_g_error.SetLogFile(ql_logFile);

		//********************************************************************
		// Check the signal manager validity
		//********************************************************************

		if (l_signal.IsBad() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProfilerCycleDecoderMain.cpp 1.5 164", 2303);
			CO_THROW(CoStc_k_error);
		}

		//********************************************************************
		// Initialize the report
		//********************************************************************

		l_reportDirectory = l_fileConf.GetValue("CO_REPORT");
		CoStc_g_report.Initialize(l_reportDirectory, qqp_argv[0]);

		//********************************************************************
		// Run the processing
		//********************************************************************

		l_showMessage = FALSE;
		l_decoder.Run(p_argc, qqp_argv);
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		if (l_showMessage == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProfilerCycleDecoderMain.cpp 1.5 191", 2304);
		}
		exit(CoStc_k_error);
	}
	exit(CoStc_k_noError);
}




