//****************************************************************************
//
// PRO : CORIOLIS
// MOD : CoDatApexSuperFrame 
// ROL : This class is designed to manage the concatenation of all the apex
//		 frame for a particular cycle
//
// CRE : 17/12/2003
// AUT : ST
// VER : @(#)1.1
//       @(#)03/12/22
//
//****************************************************************************
// 
// HST : 17/12/2003 ST  Creation
//
//****************************************************************************
//
// Copyright, IFREMER, 1999
//
//****************************************************************************
static const char CoDatApexSuperFrame_IDENT[] = "@(#) CORIOLIS : CoDatApexSuperFrame.cpp 1.1 03/12/22";


//----------------------------------------------------------------------------
// INTERFACE
//----------------------------------------------------------------------------
#define _CoDatApexSuperFrame_IN_
#include <CoDatApexSuperFrame.h>
#undef  _CoDatApexSuperFrame_IN_
#include <CoDatApexFrame.h>
#include <CoDatAttribute.h>
#include <CoDatMeasure.h>
#include <CoDatPhysicalParameterCodes.h>
#include <CoDatRecord.h>
#include <CoDatTechnicalStatus.h>
#include <CoStcCatManager.h>
#include <CoStcError.h>




//****************************************************************************
//
// SPG : CoDatApexSuperFrame
// ROL : Constructor
//
//****************************************************************************
CoDatApexSuperFrame::
CoDatApexSuperFrame(
    )
    : _isBad(TRUE)
    , _bitVec()
	, _version(0.0)
	, _ascentMeasureStartIndex(RW_NPOS)
	, _immersionMeasureStartIndex(RW_NPOS)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : ~CoDatApexSuperFrame
// ROL : Destructor
//
//****************************************************************************
CoDatApexSuperFrame::
~CoDatApexSuperFrame(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : Initialize
// ROL : Initialize the instance with the ascii frame datas
//
//****************************************************************************
void
CoDatApexSuperFrame::
Initialize(
	const RWTPtrSlist<CoDatApexFrame>& p_frames	// I : the list of frames
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWCString l_allDatas = "";				// the datas of all frames
	int l_validLength = 0;					// a frame valid length
	int l_frameCount = 0;					// number of frames
	int l_frameCounter = 0;					// loop counter
	CoDatApexFrame* ql_frame =				// the current frame
		(CoDatApexFrame*)0;
	int l_frameNumber = 0;					// the current frame number
	RWCString l_datas = "";					// the datas of a single frame

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Initialize the string containing the datas of all frames
		//********************************************************************

		l_allDatas = RWCString('F', CoDatApexSuperFrame_k_maxLength / 4);

		//********************************************************************
		// Get the normal ascii length of a frame (without header)
		//********************************************************************

		l_validLength = CoDatApexFrame::GetValidFrameLength(_version) / 4 - 4;

		//********************************************************************
		// For each frame
		//********************************************************************

		l_frameCount = p_frames.entries();
		for (l_frameCounter = 0; l_frameCounter < l_frameCount; l_frameCounter++)
		{
			//****************************************************************
			// Retrieve the frame, it's data and it's number
			//****************************************************************

			ql_frame = p_frames.at(l_frameCounter);
			l_datas = ql_frame->GetAsciiHexData();
			l_datas = l_datas(4, l_datas.length() - 4);
			l_frameNumber = ql_frame->GetFrameNumber();

			//****************************************************************
			// fill the string containg the datas of all frames
			//****************************************************************

			l_allDatas((l_frameNumber - 1) * l_validLength, l_validLength) = l_datas;
		}

		//********************************************************************
		// Initialize the bit vec
		//********************************************************************

		_bitVec.SetValues(l_allDatas);

		//********************************************************************
		// Depending of the version, compute the measure start index
		//********************************************************************

		if (_version == 1.0)
		{
			_immersionMeasureStartIndex = RW_NPOS;
			_ascentMeasureStartIndex = 192;
		}
		else if (_version == 1.1)
		{
			_immersionMeasureStartIndex = RW_NPOS;
			_ascentMeasureStartIndex = 160;
		}
		else if (_version == 4.0)
		{
			_immersionMeasureStartIndex = RW_NPOS;
			_ascentMeasureStartIndex = 240;
		}
		else if (_version == 10.0)
		{
			_immersionMeasureStartIndex = 120;
			_ascentMeasureStartIndex = 120;
			while (_bitVec.GetUInt(_ascentMeasureStartIndex, 16) != 0xdddd)
			{
				_ascentMeasureStartIndex += 48;
				if (_ascentMeasureStartIndex >= CoDatApexSuperFrame_k_maxLength)
				{
					CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 202", 2700);
					CO_THROW(CoStc_k_error);
				}
			}
			_ascentMeasureStartIndex += 128;
		}
		else if (_version == 11.0)
		{
			_immersionMeasureStartIndex = RW_NPOS;
			_ascentMeasureStartIndex = 232;
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 221", 2701);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : GetProfileLength
// ROL : Retrieve the length of the profile, ie the number of records
//
//****************************************************************************
unsigned int
CoDatApexSuperFrame::
GetProfileLength(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	unsigned int l_length = 0;			// the profile length

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if ((_version == 1.0) || (_version == 1.1) || (_version == 4.0) ||
			(_version == 10.0) || (_version == 11.0))
		{
			l_length = _bitVec.GetUInt(32, 8);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 269", 2702);
		CO_RETHROW;
	}
	return l_length;
}




//****************************************************************************
//
// SPG : GetAscentRecords
// ROL : Get the ascent records contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
GetAscentRecords(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Decod the records
		//********************************************************************

		DecodAscentRecords(p_records);
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 313", 2703);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : GetImmersionRecords
// ROL : Get the immersion records contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
GetImmersionRecords(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Decod the records
		//********************************************************************

		DecodImmersionRecords(p_records);
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 356", 2704);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : GetTechnicalStatus
// ROL : Get the data contained in a technical frame and fill the technical status
//
//****************************************************************************
void
CoDatApexSuperFrame::
GetTechnicalStatus(
	CoDatTechnicalStatus& p_status			// O : The technical status to fill
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Decod the technical status
		//********************************************************************

		DecodTechnicalStatus(p_status);
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 399", 2705);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : DecodAscentRecords
// ROL : Decod the ascent records contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
DecodAscentRecords(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if ((_version == 1.0) || (_version == 4.0) || (_version == 10.0) ||
			(_version == 11.0))
		{
			DecodAscentRecordsV10(p_records);
		}
		else if (_version == 1.1)
		{
			DecodAscentRecordsV11(p_records);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 456", 2706);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodAscentRecordsV10
// ROL : Decod the ascent records contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
DecodAscentRecordsV10(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	int l_recordCount = 0;					// number of records
	int l_recordCounter = 0;				// loop counter
	int l_field;							// a bitfield value
	double l_temperatureValue = 0.0;		// the current temperature value
	CoDatMeasure l_temperature;				// the temperature measure;
	double l_salinityValue = 0.0;			// the current salinity value
	CoDatMeasure l_salinity;				// the salinity measure;
	double l_pressureValue = 0.0;			// the current pressure value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Compute the start bit and the number of records
		//********************************************************************

		l_position = _ascentMeasureStartIndex;
		l_recordCount = GetProfileLength();

		//********************************************************************
		// For each record to decod
		//********************************************************************

		for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
		{
			//****************************************************************
			// Decod the temperature
			//****************************************************************

			l_field = _bitVec.GetUInt(l_position, 16);
			l_position += 16;
			if (l_field <= 62535)
			{
				l_temperatureValue = double(l_field) / 1000.0;
			}
			else
			{
				l_temperatureValue = -double(65536 - l_field) / 1000.0;
			}
			l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue);

			//****************************************************************
			// Decod the salinity
			//****************************************************************

			l_salinityValue = double(_bitVec.GetUInt(l_position, 16)) / 1000.0;
			l_position += 16;
			l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue);

			//****************************************************************
			// Decod the pressure
			//****************************************************************

			l_pressureValue = double(_bitVec.GetUInt(l_position, 16)) / 10.0;
			l_position += 16;
			l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue);

			//****************************************************************
			// If the pressure is decoded, create the records
			//****************************************************************

			if (l_pressureValue != 6553.5)
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				ql_record->AddMeasure(l_salinity);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 565", 2707);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodAscentRecordsV11
// ROL : Decod the ascent records contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
DecodAscentRecordsV11(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	int l_recordCount = 0;					// number of records
	int l_recordCounter = 0;				// loop counter
	int l_field;							// a bitfield value
	double l_temperatureValue = 0.0;		// the current temperature value
	CoDatMeasure l_temperature;				// the temperature measure;
	double l_salinityValue = 0.0;			// the current salinity value
	CoDatMeasure l_salinity;				// the salinity measure;
	double l_pressureValue = 0.0;			// the current pressure value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Compute the start bit and the number of records
		//********************************************************************

		l_position = _ascentMeasureStartIndex;
		l_recordCount = GetProfileLength();

		//********************************************************************
		// For each record to decod
		//********************************************************************

		for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
		{
			//****************************************************************
			// Decod the temperature
			//****************************************************************

			l_field = _bitVec.GetUInt(l_position, 16);
			l_position += 16;
			if (l_field <= 62535)
			{
				l_temperatureValue = double(l_field) / 1000.0;
			}
			else
			{
				l_temperatureValue = -double(65536 - l_field) / 1000.0;
			}
			l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue);

			//****************************************************************
			// Decod the salinity
			//****************************************************************

			l_salinityValue = double(_bitVec.GetUInt(l_position, 16)) / 10000.0 + 30.0;
			l_position += 16;
			l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue);

			//****************************************************************
			// Decod the pressure
			//****************************************************************

			l_pressureValue = double(_bitVec.GetUInt(l_position, 16)) / 10.0;
			l_position += 16;
			l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue);

			//****************************************************************
			// If the pressure is decoded, create the records
			//****************************************************************

			if (l_pressureValue != 6553.5)
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				ql_record->AddMeasure(l_salinity);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 674", 2708);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodImmersionRecords
// ROL : Decod the immersion records contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
DecodImmersionRecords(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if (_version == 10.0)
		{
			DecodImmersionRecordsV100(p_records);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 726", 2709);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodImmersionRecordsV100
// ROL : Decod the immersion records contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
DecodImmersionRecordsV100(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	int l_field;							// a bitfield value
	double l_temperatureValue = 0.0;		// the current temperature value
	CoDatMeasure l_temperature;				// the temperature measure;
	double l_salinityValue = 0.0;			// the current salinity value
	CoDatMeasure l_salinity;				// the salinity measure;
	double l_pressureValue = 0.0;			// the current pressure value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Compute the start bit and the number of records
		//********************************************************************

		l_position = _immersionMeasureStartIndex;

		//********************************************************************
		// For each record to decod
		//********************************************************************

		while (_bitVec.GetUInt(l_position, 16) != 0xdddd)
		{
			//****************************************************************
			// Decod the temperature
			//****************************************************************

			l_field = _bitVec.GetUInt(l_position, 16);
			l_position += 16;
			if (l_field <= 62535)
			{
				l_temperatureValue = double(l_field) / 1000.0;
			}
			else
			{
				l_temperatureValue = -double(65536 - l_field) / 1000.0;
			}
			l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue);

			//****************************************************************
			// Decod the salinity
			//****************************************************************

			l_salinityValue = double(_bitVec.GetUInt(l_position, 16)) / 1000.0;
			l_position += 16;
			l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue);

			//****************************************************************
			// Decod the pressure
			//****************************************************************

			l_pressureValue = double(_bitVec.GetUInt(l_position, 16)) / 10.0;
			l_position += 16;
			l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue);

			//****************************************************************
			// If the pressure is decoded, create the records
			//****************************************************************

			if (l_pressureValue != 6553.5)
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				ql_record->AddMeasure(l_salinity);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 832", 2710);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatus
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
DecodTechnicalStatus(
	CoDatTechnicalStatus& p_status			// O : the technical status
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if (_version == 1.0)
		{
			DecodTechnicalStatusV10(p_status);
		}
		else if (_version == 1.1)
		{
			DecodTechnicalStatusV11(p_status);
		}
		else if (_version == 4.0)
		{
			DecodTechnicalStatusV40(p_status);
		}
		else if (_version == 10.0)
		{
			DecodTechnicalStatusV100(p_status);
		}
		else if (_version == 11.0)
		{
			DecodTechnicalStatusV110(p_status);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 894", 2711);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV10
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
DecodTechnicalStatusV10(
	CoDatTechnicalStatus& p_status			// O : the technical status
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	double l_doubleValue = 0.0;				// A double value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// APX_GNRL_MsgBlock
		//********************************************************************

		l_intValue = _bitVec.GetUInt(0, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_MsgBlock", l_intValue));

		//********************************************************************
		// APX_GNRL_SerialNum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(8, 16);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_SerialNum", l_intValue));

		//********************************************************************
		// CYCLE_NUMBER
		//********************************************************************

		l_intValue = _bitVec.GetUInt(24, 8);
		p_status.AddAttribute(CoDatAttribute("CYCLE_NUMBER", l_intValue));

		//********************************************************************
		// APX_ASCN_ProfLength
		//********************************************************************

		l_intValue = _bitVec.GetUInt(32, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_ProfLength", l_intValue));

		//********************************************************************
		// APX_ASCN_EndFlag
		//********************************************************************

		l_intValue = _bitVec.GetUInt(40, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_EndFlag", l_intValue));

		//********************************************************************
		// APX_ASCN_PistonEndP
		//********************************************************************

		l_intValue = _bitVec.GetUInt(48, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_PistonEndP", l_intValue));

		//********************************************************************
		// APX_GNRL_AirPump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(56, 16);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_AirPump", l_intValue));

		//********************************************************************
		// APX_GNRL_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(72, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_BattVolt", l_doubleValue));

		//********************************************************************
		// APX_ASCN_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(80, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_PistonPos", l_intValue));

		//********************************************************************
		// APX_SURF_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(88, 8);
		p_status.AddAttribute(CoDatAttribute("APX_SURF_PistonPos", l_intValue));

		//********************************************************************
		// APX_GNRL_AirBladPres
		//********************************************************************

		l_intValue = _bitVec.GetUInt(96, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_AirBladPres", l_intValue));

		//********************************************************************
		// APX_PARK_Temperature
		//********************************************************************

		if (_bitVec.GetUInt(104, 16) <= 62535)
		{
			l_doubleValue = double(_bitVec.GetUInt(104, 16)) / 1000.0;
		}
		else
		{
			l_doubleValue = -double(65536 - _bitVec.GetUInt(104, 16)) / 1000.0;
		}
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Temperature", l_doubleValue));

		//********************************************************************
		// APX_PARK_Salinity
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(120, 16)) / 1000.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Salinity", l_doubleValue));

		//********************************************************************
		// APX_PARK_Pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(136, 16)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Pressure", l_doubleValue));

		//********************************************************************
		// APX_PARK_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(152, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_BattVolt", l_doubleValue));

		//********************************************************************
		// APX_SURF_Pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(160, 16)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_SURF_Pressure", l_doubleValue));

		//********************************************************************
		// APX_GNRL_IntVacuum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(176, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_IntVacuum", l_intValue));

		//********************************************************************
		// APX_PARK_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(184, 8);
		p_status.AddAttribute(CoDatAttribute("APX_PARK_PistonPos", l_intValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 1067", 2712);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV11
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
DecodTechnicalStatusV11(
	CoDatTechnicalStatus& p_status			// O : the technical status
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	double l_doubleValue = 0.0;				// A double value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// APX_GNRL_MsgBlock
		//********************************************************************

		l_intValue = _bitVec.GetUInt(0, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_MsgBlock", l_intValue));

		//********************************************************************
		// APX_GNRL_SerialNum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(8, 16);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_SerialNum", l_intValue));

		//********************************************************************
		// CYCLE_NUMBER
		//********************************************************************

		l_intValue = _bitVec.GetUInt(24, 8);
		p_status.AddAttribute(CoDatAttribute("CYCLE_NUMBER", l_intValue));

		//********************************************************************
		// APX_ASCN_ProfLength
		//********************************************************************

		l_intValue = _bitVec.GetUInt(32, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_ProfLength", l_intValue));

		//********************************************************************
		// APX_ASCN_EndFlag
		//********************************************************************

		l_intValue = _bitVec.GetUInt(40, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_EndFlag", l_intValue));

		//********************************************************************
		// APX_ASCN_PistonEndP
		//********************************************************************

		l_intValue = _bitVec.GetUInt(48, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_PistonEndP", l_intValue));

		//********************************************************************
		// APX_PARK_Temperature
		//********************************************************************

		if (_bitVec.GetUInt(56, 16) <= 62535)
		{
			l_doubleValue = double(_bitVec.GetUInt(56, 16)) / 1000.0;
		}
		else
		{
			l_doubleValue = -double(65536 - _bitVec.GetUInt(56, 16)) / 1000.0;
		}
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Temperature", l_doubleValue));

		//********************************************************************
		// APX_PARK_Salinity
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(72, 16)) / 10000.0 + 30.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Salinity", l_doubleValue));

		//********************************************************************
		// APX_PARK_Pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(88, 16)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Pressure", l_doubleValue));

		//********************************************************************
		// APX_GNRL_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(104, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_BattVolt", l_doubleValue));

		//********************************************************************
		// APX_SURF_Pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(112, 16)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_SURF_Pressure", l_doubleValue));

		//********************************************************************
		// APX_PARK_Vacuum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(128, 8);
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Vacuum", l_intValue));

		//********************************************************************
		// APX_PARK_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(136, 8);
		p_status.AddAttribute(CoDatAttribute("APX_PARK_PistonPos", l_intValue));

		//********************************************************************
		// APX_ASCN_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(144, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_PistonPos", l_intValue));

		//********************************************************************
		// APX_GNRL_IntVacuum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(152, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_IntVacuum", l_intValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 1219", 2713);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV40
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
DecodTechnicalStatusV40(
	CoDatTechnicalStatus& p_status			// O : the technical status
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	double l_doubleValue = 0.0;				// A double value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// APX_GNRL_MsgBlock
		//********************************************************************

		l_intValue = _bitVec.GetUInt(0, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_MsgBlock", l_intValue));

		//********************************************************************
		// APX_GNRL_SerialNum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(8, 16);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_SerialNum", l_intValue));

		//********************************************************************
		// CYCLE_NUMBER
		//********************************************************************

		l_intValue = _bitVec.GetUInt(24, 8);
		p_status.AddAttribute(CoDatAttribute("CYCLE_NUMBER", l_intValue));

		//********************************************************************
		// APX_ASCN_ProfLength
		//********************************************************************

		l_intValue = _bitVec.GetUInt(32, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_ProfLength", l_intValue));

		//********************************************************************
		// APX_ASCN_EndFlag
		//********************************************************************

		l_intValue = _bitVec.GetUInt(40, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_EndFlag", l_intValue));

		//********************************************************************
		// APX_ASCN_PistonEndP
		//********************************************************************

		l_intValue = _bitVec.GetUInt(48, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_PistonEndP", l_intValue));

		//********************************************************************
		// APX_GNRL_FormatNum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(56, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_FormatNum", l_intValue));

		//********************************************************************
		// APX_GNRL_DephTblNum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(64, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_DephTblNum", l_intValue));

		//********************************************************************
		// APX_GNRL_PumpMotor
		//********************************************************************

		l_intValue = _bitVec.GetUInt(72, 16);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_PumpMotor", l_intValue));

		//********************************************************************
		// APX_GNRL_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(88, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_BattVolt", l_doubleValue));

		//********************************************************************
		// APX_GNRL_BattCurr
		//********************************************************************

		l_intValue = _bitVec.GetUInt(96, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_BattCurr", l_intValue));

		//********************************************************************
		// APX_GNRL_AirPumpCurr
		//********************************************************************

		l_intValue = _bitVec.GetUInt(104, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_AirPumpCurr", l_intValue));

		//********************************************************************
		// APX_ASCN_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(112, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_PistonPos", l_intValue));

		//********************************************************************
		// APX_SURF_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(120, 8);
		p_status.AddAttribute(CoDatAttribute("APX_SURF_PistonPos", l_intValue));

		//********************************************************************
		// APX_GNRL_AirBladPres
		//********************************************************************

		l_intValue = _bitVec.GetUInt(128, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_AirBladPres", l_intValue));

		//********************************************************************
		// APX_PARK_Temperature
		//********************************************************************

		if (_bitVec.GetUInt(136, 16) <= 62535)
		{
			l_doubleValue = double(_bitVec.GetUInt(136, 16)) / 1000.0;
		}
		else
		{
			l_doubleValue = -double(65536 - _bitVec.GetUInt(136, 16)) / 1000.0;
		}
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Temperature", l_doubleValue));

		//********************************************************************
		// APX_PARK_Salinity
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(152, 16)) / 1000.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Salinity", l_doubleValue));

		//********************************************************************
		// APX_PARK_Pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(168, 16)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Pressure", l_doubleValue));

		//********************************************************************
		// APX_PARK_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(184, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_BattVolt", l_doubleValue));

		//********************************************************************
		// APX_SURF_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(192, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_SURF_BattVolt", l_doubleValue));

		//********************************************************************
		// APX_SURF_Pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(200, 16)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_SURF_Pressure", l_doubleValue));

		//********************************************************************
		// APX_GNRL_IntVacuum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(216, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_IntVacuum", l_intValue));

		//********************************************************************
		// APX_PARK_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(224, 8);
		p_status.AddAttribute(CoDatAttribute("APX_PARK_PistonPos", l_intValue));

		//********************************************************************
		// APX_GNRL_SbePumpCurr
		//********************************************************************

		l_intValue = _bitVec.GetUInt(232, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_SbePumpCurr", l_intValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 1434", 2714);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV100
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
DecodTechnicalStatusV100(
	CoDatTechnicalStatus& p_status			// O : the technical status
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	double l_doubleValue = 0.0;				// A double value
	size_t l_position = RW_NPOS;			// the position of the status 2nd part

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// APX_GNRL_MsgBlock
		//********************************************************************

		l_intValue = _bitVec.GetUInt(0, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_MsgBlock", l_intValue));

		//********************************************************************
		// APX_GNRL_SerialNum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(8, 16);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_SerialNum", l_intValue));

		//********************************************************************
		// CYCLE_NUMBER
		//********************************************************************

		l_intValue = _bitVec.GetUInt(24, 8);
		p_status.AddAttribute(CoDatAttribute("CYCLE_NUMBER", l_intValue));

		//********************************************************************
		// APX_ASCN_ProfLength
		//********************************************************************

		l_intValue = _bitVec.GetUInt(32, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_ProfLength", l_intValue));

		//********************************************************************
		// APX_ASCN_EndFlag
		//********************************************************************

		l_intValue = _bitVec.GetUInt(40, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_EndFlag", l_intValue));

		//********************************************************************
		// APX_ASCN_PistonEndP
		//********************************************************************

		l_intValue = _bitVec.GetUInt(48, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_PistonEndP", l_intValue));

		//********************************************************************
		// APX_GNRL_FormatNum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(56, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_FormatNum", l_intValue));

		//********************************************************************
		// APX_GNRL_DephTblNum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(64, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_DephTblNum", l_intValue));

		//********************************************************************
		// APX_GNRL_PumpMotor
		//********************************************************************

		l_intValue = _bitVec.GetUInt(72, 16);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_PumpMotor", l_intValue));

		//********************************************************************
		// APX_GNRL_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(88, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_BattVolt", l_doubleValue));

		//********************************************************************
		// APX_GNRL_BattCurr
		//********************************************************************

		l_intValue = _bitVec.GetUInt(96, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_BattCurr", l_intValue));

		//********************************************************************
		// APX_SURF_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(104, 8);
		p_status.AddAttribute(CoDatAttribute("APX_SURF_PistonPos", l_intValue));

		//********************************************************************
		// APX_GNRL_AirBladPres
		//********************************************************************

		l_intValue = _bitVec.GetUInt(112, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_AirBladPres", l_intValue));

		//********************************************************************
		// Compute the position of the status 2nd part
		//********************************************************************

		l_position = _ascentMeasureStartIndex - 112;

		//********************************************************************
		// APX_PARK_Temperature
		//********************************************************************

		if (_bitVec.GetUInt(l_position, 16) <= 62535)
		{
			l_doubleValue = double(_bitVec.GetUInt(l_position, 16)) / 1000.0;
		}
		else
		{
			l_doubleValue = -double(65536 - _bitVec.GetUInt(l_position, 16)) / 1000.0;
		}
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Temperature", l_doubleValue));

		//********************************************************************
		// APX_PARK_Salinity
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(l_position + 16, 16)) / 1000.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Salinity", l_doubleValue));

		//********************************************************************
		// APX_PARK_Pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(l_position + 32, 16)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Pressure", l_doubleValue));

		//********************************************************************
		// APX_BTTM_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(l_position + 48, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_BTTM_BattVolt", l_doubleValue));

		//********************************************************************
		// APX_BTTM_BattCurr
		//********************************************************************

		l_intValue = _bitVec.GetUInt(l_position + 56, 8);
		p_status.AddAttribute(CoDatAttribute("APX_BTTM_BattCurr", l_intValue));

		//********************************************************************
		// APX_SURF_Pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(l_position + 64, 16)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_SURF_Pressure", l_doubleValue));

		//********************************************************************
		// APX_GNRL_IntVacuum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(l_position + 80, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_IntVacuum", l_intValue));

		//********************************************************************
		// APX_BTTM_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(l_position + 88, 8);
		p_status.AddAttribute(CoDatAttribute("APX_BTTM_PistonPos", l_intValue));

		//********************************************************************
		// APX_GNRL_SbePumpVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(l_position + 96, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_SbePumpVolt", l_doubleValue));

		//********************************************************************
		// APX_GNRL_SbePumpCurr
		//********************************************************************

		l_intValue = _bitVec.GetUInt(l_position + 104, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_SbePumpCurr", l_intValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 1649", 2715);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV110
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatApexSuperFrame::
DecodTechnicalStatusV110(
	CoDatTechnicalStatus& p_status			// O : the technical status
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	double l_doubleValue = 0.0;				// A double value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// APX_GNRL_MsgBlock
		//********************************************************************

		l_intValue = _bitVec.GetUInt(0, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_MsgBlock", l_intValue));

		//********************************************************************
		// APX_GNRL_SerialNum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(8, 16);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_SerialNum", l_intValue));

		//********************************************************************
		// CYCLE_NUMBER
		//********************************************************************

		l_intValue = _bitVec.GetUInt(24, 8);
		p_status.AddAttribute(CoDatAttribute("CYCLE_NUMBER", l_intValue));

		//********************************************************************
		// APX_ASCN_ProfLength
		//********************************************************************

		l_intValue = _bitVec.GetUInt(32, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_ProfLength", l_intValue));

		//********************************************************************
		// APX_ASCN_EndFlag
		//********************************************************************

		l_intValue = _bitVec.GetUInt(40, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_EndFlag", l_intValue));

		//********************************************************************
		// APX_ASCN_PistonEndP
		//********************************************************************

		l_intValue = _bitVec.GetUInt(48, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_PistonEndP", l_intValue));

		//********************************************************************
		// APX_GNRL_FormatNum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(56, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_FormatNum", l_intValue));

		//********************************************************************
		// APX_GNRL_DephTblNum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(64, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_DephTblNum", l_intValue));

		//********************************************************************
		// APX_GNRL_PumpMotor
		//********************************************************************

		l_intValue = _bitVec.GetUInt(72, 16);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_PumpMotor", l_intValue));

		//********************************************************************
		// APX_GNRL_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(88, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_BattVolt", l_doubleValue));

		//********************************************************************
		// APX_GNRL_BattCurr
		//********************************************************************

		l_intValue = _bitVec.GetUInt(96, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_BattCurr", l_intValue));

		//********************************************************************
		// APX_ASCN_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(104, 8);
		p_status.AddAttribute(CoDatAttribute("APX_ASCN_PistonPos", l_intValue));

		//********************************************************************
		// APX_GNRL_AirBladPres
		//********************************************************************

		l_intValue = _bitVec.GetUInt(112, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_AirBladPres", l_intValue));

		//********************************************************************
		// APX_PARK_Temperature
		//********************************************************************

		if (_bitVec.GetUInt(120, 16) <= 62535)
		{
			l_doubleValue = double(_bitVec.GetUInt(120, 16)) / 1000.0;
		}
		else
		{
			l_doubleValue = -double(65536 - _bitVec.GetUInt(120, 16)) / 1000.0;
		}
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Temperature", l_doubleValue));

		//********************************************************************
		// APX_PARK_Salinity
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(136, 16)) / 1000.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Salinity", l_doubleValue));

		//********************************************************************
		// APX_PARK_Pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(152, 16)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_Pressure", l_doubleValue));

		//********************************************************************
		// APX_PARK_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(168, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_PARK_BattVolt", l_doubleValue));

		//********************************************************************
		// APX_PARK_BattCurr
		//********************************************************************

		l_intValue = _bitVec.GetUInt(176, 8);
		p_status.AddAttribute(CoDatAttribute("APX_PARK_BattCurr", l_intValue));

		//********************************************************************
		// APX_SURF_Pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(184, 16)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_SURF_Pressure", l_doubleValue));

		//********************************************************************
		// APX_GNRL_IntVacuum
		//********************************************************************

		l_intValue = _bitVec.GetUInt(200, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_IntVacuum", l_intValue));

		//********************************************************************
		// APX_PARK_PistonPos
		//********************************************************************

		l_intValue = _bitVec.GetUInt(208, 8);
		p_status.AddAttribute(CoDatAttribute("APX_PARK_PistonPos", l_intValue));

		//********************************************************************
		// APX_GNRL_SbePumpVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(216, 8)) / 10.0;
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_SbePumpVolt", l_doubleValue));

		//********************************************************************
		// APX_GNRL_SbePumpCurr
		//********************************************************************

		l_intValue = _bitVec.GetUInt(224, 8);
		p_status.AddAttribute(CoDatAttribute("APX_GNRL_SbePumpCurr", l_intValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexSuperFrame.cpp 1.1 1857", 2716);
		CO_RETHROW;
	}
}








