//****************************************************************************
//
// PRO : CORIOLIS
// MOD : CoDatDataFrame 
// ROL : This class is the base class for all the data frames classes. It
//       provides several pure virtual member functions which should be
//       overidden in inherited classes.
//
// CRE : 28/08/1999
// AUT : ST
// VER : @(#)1.3
//       @(#)02/08/27
//****************************************************************************
// 
// HST : 28/08/1999 ST  Creation
//		 27/08/2002 ST  Now manage the emitting date-time and satellite
//
//****************************************************************************
//
// Copyright, IFREMER, 1999
//
//****************************************************************************
static const char CoDatDataFrame_IDENT[] = "@(#) CORIOLIS : CoDatDataFrame.cpp 1.3 02/08/27";


//----------------------------------------------------------------------------
// INTERFACE
//----------------------------------------------------------------------------
#define _CoDatDataFrame_IN_
#include <CoDatDataFrame.h>
#undef  _CoDatDataFrame_IN_
#include <CoStcError.h>




//****************************************************************************
//
// SPG : CoDatDataFrame
// ROL : Constructor
//
//****************************************************************************
CoDatDataFrame::
CoDatDataFrame(
    )
    : _isBad(TRUE)
    , _emittingSatellite('\0')
	, _emittingDateTime((unsigned long)0)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
	CoStc_g_error.Send(3, "CoDatDataFrame.cpp 1.3 68", 201);
	_isBad = TRUE;
    }
}




//****************************************************************************
//
// SPG : ~CoDatDataFrame
// ROL : Destructor
//
//****************************************************************************
CoDatDataFrame::
~CoDatDataFrame(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : CompareEmittingDateTimeAndSatellite
// ROL : Compare two Apex frames emitting date/time and emitting satellite
//		 -1 if 1st < 2nd
//		 0 if 1st = 2nd
//		 1 if 1st > 2nd
//
//****************************************************************************
/*static*/ int
CoDatDataFrame::
CompareEmittingDateTimeAndSatellite(
	void* qp_1stFrame,						// I : the first frame
	void* qp_2ndFrame,						// I : the second frame
	void*									// N : not used
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWTime l_1stFrameDateTime = 			// the first frame's date/time
		RWTime((unsigned long)0);
	RWTime l_2ndFrameDateTime = 			// the second frame's date/time
		RWTime((unsigned long)0);
	char l_1stFrameSatellite = '\0';		// the first frame's satellite
	char l_2ndFrameSatellite = '\0';		// the second frame's satellite
	int l_compare = 0;						// the comparison result

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve each frame's date/time
		//********************************************************************

		l_1stFrameDateTime = ((CoDatDataFrame*)qp_1stFrame)->GetEmittingDateTime();
		l_2ndFrameDateTime = ((CoDatDataFrame*)qp_2ndFrame)->GetEmittingDateTime();

		//********************************************************************
		// Retrieve each frame's satellite
		//********************************************************************

		l_1stFrameSatellite = ((CoDatDataFrame*)qp_1stFrame)->GetEmittingSatellite();
		l_2ndFrameSatellite = ((CoDatDataFrame*)qp_2ndFrame)->GetEmittingSatellite();

		//********************************************************************
		// And compare them
		//********************************************************************

		if (l_1stFrameDateTime < l_2ndFrameDateTime)
		{
			l_compare = -1;
		}
		else if (l_1stFrameDateTime == l_2ndFrameDateTime)
		{
			if (l_1stFrameSatellite < l_2ndFrameSatellite)
			{
				l_compare = -1;
			}
			else if (l_1stFrameSatellite == l_2ndFrameSatellite)
			{
				l_compare = 0;
			}
			else
			{
				l_compare = 1;
			}
		}
		else
		{
			l_compare = 1;
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatDataFrame.cpp 1.3 187", 9999);
		CO_RETHROW;
	}
	return l_compare;
}








